/***************************************************************************
 *   Copyright (C) 2001,2002,2003,2004,2005,2006,2007,2008,2009            *
 *   by Rick L. Vinyard, Jr.                                               *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUS_IPV6UDP_H
#define CONEXUS_IPV6UDP_H

#include <set>

#ifdef CONEXUS_HAVE_LINUX_UDP_H
#include <linux/udp.h>
#endif

#include <conexus/ipv6_ip.h>

/**
 * \example ipv6_udp_client.cpp
 * This is an example of how to use the Conexus::IPv6::UDP class.
 *
 * \example ipv6_udp_server.cpp
 * This is an example of how to use the Conexus::IPv6::UDP class.
 *
 * \example ipv6_multicast_client.cpp
 * This is an example of how to use multicasting with the Conexus::IPv6::UDP class.
 *
 * \example ipv6_multicast_server.cpp
 * This is an example of how to use multicasting with the Conexus::IPv6::UDP class.
 *
 */

namespace Conexus
{

  namespace IPv6
  {

    /**
     * IPv6 UDP Endpoint
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup IPv6
     * @ingroup Endpoint
     *
     * @TODO TODO Fix multicast loopback as in IPv4::UDP
     */
    class UDP : public IP
    {
      protected:

        UDP( const struct in6_addr& local_if, uint16_t localport, uint16_t remoteport, int proto = IPPROTO_UDP );

        UDP( const std::string& local_if, uint16_t localport, uint16_t remoteport, int proto = IPPROTO_UDP );

      public:

        typedef ConexusPointer<UDP> pointer;

        CONEXUS_ENDPOINT_GENERIC_CREATE();

        /**
         * Creates a UDP endpoint on the default local interface with the given local
         * and remote ports
         */
        static pointer create( uint16_t localport = 0, uint16_t remoteport = 0 );

        /**
         * Creates a UDP endpoint on the specified local interface with the given
         * local and remote ports
         */
        static pointer create( const struct in6_addr& local_if, uint16_t localport = 0, uint16_t remoteport = 0 );

        /**
         * Creates a UDP endpoint on the named local interface with the given
         * local and remote ports
         */
        static pointer create( const std::string& local_if, uint16_t localport = 0, uint16_t remoteport = 0 );

        /** Automatically leaves any multicast groups this endpoint has joined */
        virtual ~UDP() throw();

        /** Returns the amount of input currently available for reading */
        virtual size_t input_available() throw();

        /** Binds the UDP endpoint to the local address */
        virtual void bind() throw ( bind_exception );

        /** Sets the local address to the given parameter and binds the endpoint */
        virtual void bind( const Conexus::Address& a ) throw ( bind_exception );

        /** Closes the endpoint */
        virtual void close() throw ( close_exception );

        /**
         * Returns the size in bytes of the local send queue.
         */
        virtual int output_pending();

        /**
         * Connects the endpoint to the currently set remote address
         */
        virtual void connect() throw ( connect_exception );

        /**
         * Sets the remote address to the given parameter and connects to the given address
         */
        virtual void connect( const Address& a ) throw ( connect_exception );

        /**
         * There are some circumstances in which it may be desirable to set a
         * default address and have the send method transparently call the sendto method
         * without connecting rather than connecting and calling the send method.
         *
         * A specific example of this is when the remote address/port may occassionally
         * be unavailable. In this case the destination may generate an ICMP UNREACHABLE
         * message, and in accordance with RFC 1122 the local machine will fail any
         * future sends on a connected port, throwing a write::connection_refused exception.
         * If you want to ensure that the datagram is sent contrary to RFC 1122 you could
         * set the default address, set send without connect and call send without throwing
         * connection exceptions.
         *
         * But, since this is contrary to RFC 1122 behavior you should really think about
         * your design before setting this.
         */
        virtual void set_write_without_connect( bool value = true );

        virtual bool write_without_connect() const;

        void add_multicast_interface( unsigned iface_index );

        void add_multicast_interface( const std::string& iface_name );

        void remove_multicast_interface( unsigned iface_index );

        void remove_multicast_interface( const std::string& iface_name );

        bool is_multicast_interface( unsigned iface_index );

        bool is_multicast_interface( const std::string& iface_name );

        virtual bool suppress_multicast_join() const;

        virtual void set_suppress_multicast_join( bool value = true );

        /**
         * Returns the multicast ttl of the currently associated address.
         */
        virtual int multicast_hop_limit();

        /**
         * Sets the multicast ttl of the currently associated address to @em hop_limit.
         */
        virtual void set_multicast_hop_limit( int hop_limit );

        static unsigned int default_multicast_hop_limit();

        static void set_default_multicast_hop_limit(unsigned int hop_limit);

        /**
         * If true multicast packets are looped back to local sockets.
         */
        virtual bool is_multicast_loopback_enabled();

        /**
         * If parameter @em enable is true then multicast packets are looped back to local sockets.
         */
        virtual void enable_multicast_loopback( bool enable );

        static bool is_default_multicast_loopback_enabled();

        static void enable_default_multicast_loopback(bool enable);

        #ifdef UDP_CORK
        /**
         * All data output on this socket is accumulated into a single datagram that is transmitted
         * when the socket is uncorked.
         */
        void cork();

        /**
         * All data output on this socket is accumulated into a single datagram that is transmitted
         * when the socket is uncorked.
         */
        void uncork();

        bool is_corked();
        #endif

      protected:

        static unsigned int m_default_hop_limit;
        static bool m_default_loopback_enabled;

        bool m_write_without_connect;
        bool m_suppress_multicast_join;
        std::set<unsigned> m_multicast_interfaces;

        virtual void on_local_address_changed( );
        virtual void on_remote_address_changed( );

        /**
         * Issues a join on all multicast interfaces.
         * If no interfaces have been specified, then the join is issued on the
         * system default.
         */
        virtual void join();

        /**
         * Issues a join on the local address for the specified interface.
         */
        virtual void join(unsigned interface);

        /**
         * Issues a leave on all multicast interfaces.
         * If no interfaces have been specified, then the leave is issued on the
         * system default.
         */
        virtual void leave();

        /**
         * Issues a leave on the local address for the specified interface.
         */
        virtual void leave(unsigned interface);

        /**
         * Overloads the Socket ancestor send method to ensure that the socket is connected
         * before trying to send.
         */
        virtual size_t write_data( const Data data, Timeout timeout ) throw ( write_exception );
    };

  }

}

#endif
